%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gary Poon
% Ryan Wilshusen
%
% Red Eye Reduction Project
% This script finds and fixes redeyes using the Smolka algorithm of red eye
% enhancement
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
clear all;

orig_image = imread('redeye.jpg'); % change the filename to change the image to be processed


norm_orig_image = double(orig_image)/255;
% norm_orig_image = norm_orig_image .^ 0.9; % Attenuating eyes (0.3)

figure;
imshow(norm_orig_image);
title('Original Image');

% yiq_image = rgb2ntsc(norm_orig_image); %convert to YIQ -- enhancement = YUV
yuv_image = colorspace('yuv<-rgb',norm_orig_image);

[width length color_height] = size(orig_image); %image dimensions

r_image=orig_image(:, :, 1); %red image
g_image=orig_image(:, :, 2); %green image
b_image=orig_image(:, :, 3); %blue image

y_image=yuv_image(:, :, 1); %luminance image
u_image=yuv_image(:, :, 2); %hue image
v_image=yuv_image(:, :, 3); %saturation image

red_detect_image = u_image;

for index_x = 1:width
    for index_y = 1:length
        red_detect_image(index_x, index_y, :) = (((norm_orig_image(index_x,index_y,1) - max(norm_orig_image(index_x,index_y,2), norm_orig_image(index_x,index_y,3)))^2)/(norm_orig_image(index_x,index_y,1)+0.01));
        % red_detect_image(index_x, index_y, :) = 0;
    end
end

figure;
imshow(red_detect_image);
title('Red Enhancement');
% imwrite(red_detect_image, 'Smolka.jpg', 'quality', 100);


red_detect_edge = edge(red_detect_image, 'canny', 0.5);

figure;
imshow(red_detect_edge);
title('Edges of red enhancement image');
imwrite(red_detect_edge, '1.jpg', 'quality', 100);

radius = 1;
se = strel('disk', radius);
red_detect_edge = imclose(red_detect_edge, se);

% figure;
% imshow(red_detect_edge);

[y0detect,x0detect,Accumulator] = houghcircle(red_detect_edge,2,10);

circle_detect = red_detect_edge;

for index_x = 1:width
    for index_y = 1:length
        circle_detect(index_x,index_y,:) = 0;
    end
end

for i = 1:max(size(x0detect))
    circle_detect(y0detect(i),x0detect(i),:) = 1;
end

circle_detect = bwmorph(circle_detect,'thicken',10);


figure;
imshow(circle_detect);
title('Areas that contain circular edges');
imwrite(circle_detect, '2.jpg', 'quality', 100);

mask_image = red_detect_image;


threshold = 0.275;

for index_x = 1:width
    for index_y = 1:length
        if red_detect_image(index_x, index_y, :) >= threshold && circle_detect(index_x, index_y, :) == 1
            mask_image(index_x, index_y, :) = 1;
        else
            mask_image(index_x, index_y, :) = 0;
        end
    end
end



% mask_image = im2bw(red_detect_image, threshold);

radius = 2;
se = strel('disk', radius);
mask_image = imclose(mask_image, se);
% h = fspecial('gaussian', [5, 5], 10);
h = fspecial('disk', 1);
mask_image = imfilter(mask_image, h, 'circular', 'same');

% imwrite(mask_image, 'mask_image1.jpg', 'quality', 100);

figure;
imshow(mask_image);
title('Correction Factor Image');
imwrite(mask_image, '3.jpg', 'quality', 100);

bw_image = rgb2gray(norm_orig_image);

for index_x = 1:width
    for index_y = 1:length
        norm_orig_image(index_x, index_y, 1) = (1 - mask_image(index_x, index_y)) * norm_orig_image(index_x, index_y, 1) + (mask_image(index_x, index_y)) * bw_image(index_x, index_y);
        norm_orig_image(index_x, index_y, 2) = (1 - mask_image(index_x, index_y)) * norm_orig_image(index_x, index_y, 2) + (mask_image(index_x, index_y)) * bw_image(index_x, index_y);
        norm_orig_image(index_x, index_y, 3) = (1 - mask_image(index_x, index_y)) * norm_orig_image(index_x, index_y, 3) + (mask_image(index_x, index_y)) * bw_image(index_x, index_y);
    end
end

figure;
imshow(norm_orig_image);
title('Corrected Image');
% imwrite(norm_orig_image, 'corrected3.jpg', 'quality', 100);




